# -*- coding: utf-8 -*-

"""Top-level package for potranslator."""

__author__ = """SekouD"""
__email__ = 'sekoud.python@gmail.com'
__version__ = '1.0.4'
__copyright__ = "Copyright (c) 2017, SekouD"
__credits__ = ("SekouD",)
__license__ = "BSD"
__maintainer__ = "SekouD"
__status__ = "Production"

import json
import polib
from googletrans import Translator
import pkg_resources
import platform
from locale import windows_locale, getdefaultlocale
import gettext
import inspect
from codecs import open

# Sets up the automatic translation of annotated strings displayed to the user.
_RESOURCE_PACKAGE = __name__
_TRANSLATIONS_PATH = pkg_resources.resource_filename(_RESOURCE_PACKAGE, 'locale')

json_file = pkg_resources.resource_filename(_RESOURCE_PACKAGE, 'supported_languages.json')

with open(json_file, 'r', encoding='utf-8') as file:
    SUPPORTED_LANGUAGES = json.load(file)

_TRANSLATED_LANGUAGES = [key for key in SUPPORTED_LANGUAGES if key != 'en']


def _get_user_locale():
    """
    | Gets the user locale to set the user interface language language.
    | The default is set to english if the user's system locale is not one of the translated languages.

    :return: string.
        The user locale.

    """
    if 'Windows' in platform.system():
        import ctypes
        windll = ctypes.windll.kernel32
        default_locale = windows_locale[windll.GetUserDefaultUILanguage()]
    else:
        default_locale = getdefaultlocale()
    if default_locale:
        if isinstance(default_locale, tuple):
            user_locale = [0][:2]
        else:
            user_locale = default_locale[:2]
    else:
        user_locale = 'en'
    return user_locale


def _getdoc(object):
    """
    Translates the docstrings of the objects defined in the packeage in the supported languages.

    :param object:
    :return: string.
        The translated version of the object's docstring.
    """
    try:
        doc = object.__doc__
    except AttributeError:
        return None
    if not isinstance(doc, str):
        return None
    return inspect.cleandoc(_(doc))


_user_locale = _get_user_locale()

if _user_locale in _TRANSLATED_LANGUAGES:
    _POTRANSLATOR_TRANSLATIONS = gettext.translation(domain='potranslator',
                                                localedir=_TRANSLATIONS_PATH,
                                                languages=[_user_locale], fallback=True, codeset='UTF-8')
else:
    _POTRANSLATOR_TRANSLATIONS = gettext.NullTranslations()

_POTRANSLATOR_TRANSLATIONS.install()


# Replaces the getdoc method
inspect.getdoc = _getdoc

from .potranslator import PoTranslator



#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""Tests for `potranslator` package."""

import pytest
import sys
import os
import re
from textwrap import dedent
from os.path import isfile, join, getmtime
from os import remove

from click.testing import CliRunner


runner = CliRunner()

from path import Path as path

from potranslator import PoTranslator
from potranslator import commands


is_appveyor = 'APPVEYOR' in os.environ
is_travis = 'TRAVIS' in os.environ
is_CI = is_travis or is_appveyor
is_python2 = sys.version_info < (3, 0)
__dir__ = path(os.path.dirname('./tests/'))


@pytest.fixture(scope="function")
def temp(request, tmpdir):
    template_dir = 'root'

    tmpdir = path(tmpdir)
    (__dir__ / template_dir).copytree(tmpdir / template_dir)
    cwd = os.getcwd()
    temp = tmpdir / template_dir
    os.chdir(temp)

    def fin():
        os.chdir(cwd)
    request.addfinalizer(fin)
    return temp


@pytest.fixture(scope="function")
def home_in_temp(request, tmpdir):
    """change HOME environment variable to temporary location

     To avoid real .transifexrc will be rewritten.
     """
    home = os.environ.get('HOME')
    os.environ['HOME'] = tmpdir.strpath

    def fin():
        del os.environ['HOME']
        if home:
            os.environ['HOME'] = home
    request.addfinalizer(fin)
    return tmpdir


class TestPoTranslator:
    translator = PoTranslator(pot_dir='./tests/test_data/test_pot_files', locale_dir='./tests/test_data/locale')
    test_po_file = './tests/test_data/empty_test.po'
    test_pot_file = './tests/test_data/test_pot_files/test-usage.pot'
    test_languages = ('es', 'fr', 'it', 'pt', 'ro')

    def test_failed_translate_unsupported_language(self):
        """

        :return:
        """
        with pytest.raises(ValueError):
            failed_translation = self.translator.translate(self.test_po_file, 'sp')
        return

    def test_failed_translate_auto_detect(self):
        """

        :return:
        """
        with pytest.raises(ValueError):
            failed_translation = self.translator.translate(self.test_po_file)
        return

    def test_translate(self):
        """

        :return:
        """
        translation, updated = self.translator.translate(self.test_po_file, 'es')
        assert all([entry.msgstr != '' for entry in translation])
        if not is_python2:
            assert translation[0].msgstr == 'Créditos'
        else:
            assert translation[0].msgstr.encode('utf-8') == 'Créditos'
        return

    def test_translate_auto_save(self):
        """

        :return:
        """
        modif_time = getmtime(self.test_po_file)
        translation, updated = self.translator.translate(self.test_po_file, 'es', auto_save=True)
        last_modif_time = getmtime(self.test_po_file)
        assert modif_time < last_modif_time
        return

    def test_translate_all_locale(self):
        """

        :return:
        """
        translations = self.translator.translate_all_locale()
        assert not 'sp' in translations
        assert all(k in translations for k in self.test_languages)
        assert all('authors.po' in k for k in list(translations.values()))
        if not is_python2:
            assert translations['es']['authors.po'][0].msgstr == 'Créditos'
        else:
            assert translations['es']['authors.po'][0].msgstr.encode('utf-8') == 'Créditos'
        return

    def test_translate_from_pot(self):
        """

        :return:
        """
        status = {
            'created': 0,
            'updated': 0,
            'not_changed': 0,
        }
        translations = self.translator.translate_from_pot(self.test_pot_file, status, target_langs=self.test_languages)
        assert all(k in translations for k in self.test_languages)
        if not is_python2:
            assert translations['es'][0].msgstr == 'Uso'
        else:
            assert translations['es'][0].msgstr.encode('utf-8') == 'Uso'
        assert isfile(join(self.translator.locale_dir, '/'.join(('es', 'LC_MESSAGES', 'test-usage.po'))))
        for locale in self.test_languages:
            remove(join(self.translator.locale_dir, '/'.join((locale, 'LC_MESSAGES', 'test-usage.po'))))
        return

    def test_translate_all_pot(self):
        """

        :return:
        """
        translations = self.translator.translate_all_pot(target_langs=self.test_languages)
        assert all(k in translations['test-authors.pot'] for k in self.test_languages)
        assert all(k in translations['test-usage.pot'] for k in self.test_languages)

        if not is_python2:
            assert translations['test-authors.pot']['es'][0].msgstr == 'Créditos'
        else:
            assert translations['test-authors.pot']['es'][0].msgstr.encode('utf-8') == 'Créditos'
        for locale in self.test_languages:
            for file in ('test-authors.po', 'test-usage.po'):
                remove(join(self.translator.locale_dir, '/'.join((locale, 'LC_MESSAGES', file))))
        return


class TestCommandLine:
    def test_command_line_interface(self):
        """Test the CLI."""
        result = runner.invoke(commands.main)
        assert result.exit_code == 0
        assert 'POTRANSLATOR_<UPPER_LONG_NAME>' in result.output
        help_result = runner.invoke(commands.main, ['--help'])
        assert help_result.exit_code == 0
        assert 'All command-line options can be set' in help_result.output

    def test_update_pot_notfound(self, temp):
        r1 = runner.invoke(commands.update, ['-d', 'locale'])
        assert r1.exit_code != 0
        assert 'Please specify a pot directory with -p option,' in r1.output

    def test_update_no_language(self, temp):
        r1 = runner.invoke(commands.update, ['-d', 'locale', '-p', '_build/locale'])
        assert r1.exit_code != 0
        assert 'No languages were found. Please specify a language with -l' in r1.output

    def test_update_simple(self, temp):
        r1 = runner.invoke(commands.update, ['-d', 'locale', '-p', '_build/locale', '-l', 'ja'])
        assert r1.exit_code == 0

    def test_update_difference_detect(self, temp):
        r1 = runner.invoke(commands.update, ['-d', 'locale', '-p', '_build/locale', '-l', 'ja'])
        assert r1.exit_code == 0
        assert r1.output.count('Created:') == 1
        assert r1.output.count('Updated:') == 1
        assert r1.output.count('Not Changed:') == 0

        with open('_build/locale/README.pot', 'a') as f:
            f.write('\nmsgid "test1"\nmsgstr ""\n')

        r2 = runner.invoke(commands.update, ['-d', 'locale', '-p', '_build/locale'])
        assert r2.exit_code == 0
        assert r2.output.count('Created:') == 0
        assert r2.output.count('Updated:') == 0
        assert r2.output.count('Not Changed:') == 1

        with open('_build/locale/README.pot', 'r') as f:
            d = f.read()
            d = d.replace('test1', 'test2')
        with open('_build/locale/README.pot', 'w') as f:
            f.write(d)

        r3 = runner.invoke(commands.update, ['-d', 'locale', '-p', '_build/locale'])
        assert r3.exit_code == 0
        assert r3.output.count('Create:') == 0
        assert r3.output.count('Update:') == 0
        assert r3.output.count('Not Changed:') == 1

        r4 = runner.invoke(commands.update, ['-d', 'locale', '-p', '_build/locale'])
        assert r4.exit_code == 0
        assert r4.output.count('Create:') == 0
        assert r4.output.count('Update:') == 0
        assert r4.output.count('Not Changed:') == 1

    # def test_update_with_conf_file(self, temp):
    #     r1 = runner.invoke(commands.update, ['-d', 'locale', '-p', '_build/locale', '-l', 'ja'])
    #     assert r1.exit_code == 0

    def test_stat(self, temp):
        r1 = runner.invoke(commands.update, ['-d', 'locale', '-p', '_build/locale', '-l', 'ja'])
        assert r1.exit_code == 0

        r2 = runner.invoke(commands.stat, ['-d', 'locale'])
        assert r2.exit_code == 0
        assert 'README.po: 1 translated, 0 fuzzy, 0 untranslated.' in r2.output

    def test_stat_with_multiple_languages(self, temp):
        r1 = runner.invoke(commands.update, ['-d', 'locale', '-p', '_build/locale', '-l', 'ja,de,it'])
        assert r1.exit_code == 0

        # r2 = runner.invoke(commands.stat, ['-d', 'locale', '-l', 'ja,de', '-l', 'it'])
        r2 = runner.invoke(commands.stat, ['-d', 'locale', '-l', 'ja'])
        assert r2.exit_code == 0
        assert 'README.po: 1 translated, 0 fuzzy, 0 untranslated.' in r2.output

    def test_build(self, temp):
        result = runner.invoke(commands.build, ['--locale-dir', 'locale'])
        assert result.exit_code == 0


class TestTransifexCMD:
    def test_create_transifexrc(self, home_in_temp):
        r1 = runner.invoke(commands.main,
                           [
                               'create-transifexrc',
                               '--transifex-username', 'spam-id',
                               '--transifex-password', 'egg-pw',
                           ])
        assert r1.exit_code == 0

    def test_create_txconfig(self, home_in_temp, temp):
        r1 = runner.invoke(commands.main, ['create-txconfig'])
        assert r1.exit_code == 0

    @pytest.mark.skipif(is_CI, reason="Temp folder shenanigans.")
    def test_update_txconfig_resources(self, home_in_temp, temp):
        r1 = runner.invoke(commands.main, ['create-txconfig'])
        assert r1.exit_code == 0
        r2 = runner.invoke(commands.update_txconfig_resources,
                           [
                               '--transifex-project-name', 'ham-project',
                               '-d', 'locale',
                           ])
        assert r2.exit_code == 0

    @pytest.mark.skipif(is_CI, reason="Temp folder shenanigans.")
    def test_update_txconfig_resources_with_config(self, home_in_temp, temp):
        tx_dir = temp / '.tx'
        tx_dir.makedirs()
        (tx_dir / 'config').write_text(dedent("""\
        [main]
        host = https://www.transifex.com

        [ham-project.domain1]
        """))

        (temp / '_build' / 'locale').copytree(temp / 'locale' / 'pot')

        r1 = runner.invoke(commands.main, ['update-txconfig-resources', '-d', 'locale'])
        assert r1.exit_code == 0

        data = (tx_dir / 'config').text()
        assert re.search(r'\[ham-project\.README\]', data)

    @pytest.mark.skipif(is_CI, reason="Temp folder shenanigans.")
    def test_update_txconfig_resources_with_pot_dir_argument(self, home_in_temp, temp):
        tx_dir = temp / '.tx'
        tx_dir.makedirs()
        (tx_dir / 'config').write_text(dedent("""\
        [main]
        host = https://www.transifex.com

        [ham-project.domain1]
        """))

        r1 = runner.invoke(commands.main,
                           ['update-txconfig-resources',
                            '-d', 'locale',
                            '-p', '_build/locale',
                            ])
        assert r1.exit_code == 0

        data = (tx_dir / 'config').text().replace('\\', '/')
        assert re.search(r'\[ham-project\.README\]', data)
        assert re.search(r'source_file = _build/locale/README.pot', data)

    @pytest.mark.skipif(is_CI, reason="Temp folder shenanigans.")
    def test_update_txconfig_resources_with_project_name_including_dots(self, home_in_temp, temp):
        tx_dir = temp / '.tx'
        tx_dir.makedirs()
        (tx_dir / 'config').write_text(dedent("""\
        [main]
        host = https://www.transifex.com
        """))

        (temp / '_build' / 'locale').copytree(temp / 'locale' / 'pot')

        r1 = runner.invoke(commands.main,
                           ['update-txconfig-resources',
                            '-d', 'locale',
                            '--transifex-project-name', 'ham-project.com',
                            ])
        assert r1.exit_code == 0

        data = (tx_dir / 'config').text()
        assert re.search(r'\[ham-projectcom\.README\]', data)

    @pytest.mark.skipif(is_CI, reason="Temp folder shenanigans.")
    def test_update_txconfig_resources_with_project_name_including_spaces(self, home_in_temp, temp):
        tx_dir = temp / '.tx'
        tx_dir.makedirs()
        (tx_dir / 'config').write_text(dedent("""\
        [main]
        host = https://www.transifex.com
        """))

        (temp / '_build' / 'locale').copytree(temp / 'locale' / 'pot')

        r1 = runner.invoke(commands.main,
                           ['update-txconfig-resources',
                            '-d', 'locale',
                            '--transifex-project-name', 'ham project com',
                            ])
        assert r1.exit_code == 0

        data = (tx_dir / 'config').text()
        assert re.search(r'\[ham-project-com\.README\]', data)

    @pytest.mark.skipif(is_CI, reason="Temp folder shenanigans.")
    def test_update_txconfig_resources_with_potfile_including_symbols(self, home_in_temp, temp):
        tx_dir = temp / '.tx'
        tx_dir.makedirs()
        (tx_dir / 'config').write_text(dedent("""\
        [main]
        host = https://www.transifex.com
        """))

        (temp / '_build' / 'locale').copytree(temp / 'locale' / 'pot')

        # copy README.pot to 'example document.pot'
        readme = (temp / '_build' / 'locale' / 'README.pot').text()
        (temp / '_build' / 'locale' / 'example document.pot').write_text(readme)

        # copy README.pot to 'test.document.pot'
        (temp / '_build' / 'locale' / 'test.document.pot').write_text(readme)

        r1 = runner.invoke(commands.main,
                           ['update-txconfig-resources',
                            '-d', 'locale',
                            '--transifex-project-name', 'ham project com',
                            ])
        assert r1.exit_code == 0

        data = (tx_dir / 'config').text()
        assert re.search(r'\[ham-project-com\.example_document\]', data)
        assert re.search(r'\[ham-project-com\.test_document\]', data)



# -*- coding: utf-8 -*-
"""
    potranslator
    ~~~~~~~~~~~
    Python package to easily translate po and pot files in any language supported by Google Translate.
    Command Line Script adapted from sphinx-intl for compatibility.

    :copyright: Copyright (c) 2018 by SekouD.
    :copyright: Copyright (c) 2013 by Takayuki SHIMIZUKAWA.
    :license: BSD, see LICENSE for details.
"""

import os
from glob import glob

import click

# from . import catalog as c
from .pycompat import relpath
from .potranslator import PoTranslator
from . import polib


# ==================================
# utility functions

def get_lang_dirs(path):
    dirs = [relpath(d, path)
            for d in glob(path + '/[a-z]*')
            if os.path.isdir(d) and not d.endswith('pot')]
    return (tuple(dirs),)


# ==================================
# commands

def update(locale_dir, pot_dir, languages):
    """
    Update specified language's po files from pot.

    :param unicode locale_dir: path for locale directory
    :param unicode pot_dir: path for pot directory
    :param tuple languages: languages to update po files
    :return: Dict of POFiles.
    :rtype: dict
    """
    translator = PoTranslator(pot_dir=pot_dir, locale_dir=locale_dir)
    results = translator.translate_all_pot(target_langs=languages, auto_save=True)
    return results


def build(locale_dir, output_dir, languages):
    """
    Biuilds specified language's mo files from pot.

    :param unicode locale_dir: path for locale directory
    :param unicode output_dir: path for mo output directory
    :param tuple languages: languages to update po files
    :return: Dict of POFiles.
    :rtype: dict
    """
    translator = PoTranslator(pot_dir=output_dir, locale_dir=locale_dir)
    results = translator.translate_all_pot(target_langs=languages, auto_save=True, compiled=True)
    return results


def stat(locale_dir, languages):
    """
    Print statistics for all po files.

    :param unicode locale_dir: path for locale directory
    :param tuple languages: languages to update po files
    :return: {'FILENAME': {'translated': 0, 'fuzzy': 0, 'untranslated': 0}, ...}
    :rtype: dict
    """
    result = {}

    for lang in languages:
        lang_dir = os.path.join(locale_dir, lang)
        for dirpath, dirnames, filenames in os.walk(lang_dir):
            for filename in filenames:
                po_file = os.path.join(dirpath, filename)
                base, ext = os.path.splitext(po_file)
                if ext != ".po":
                    continue

                cat = polib.pofile(po_file)
                r = result[po_file.replace('\\', '/')] = {
                    'translated': len(cat.translated_entries()),
                    'fuzzy': len(cat.fuzzy_entries()),
                    'untranslated': len(cat.untranslated_entries()),
                }
                click.echo(
                    '{0}: {1} translated, {2} fuzzy, {3} untranslated.'.format(
                        po_file,
                        r['translated'],
                        r['fuzzy'],
                        r['untranslated'],
                    )
                )

    return result



# -*- coding: utf-8 -*-

"""Main module."""

from os import listdir, makedirs
from os.path import isfile, join, exists
from . import polib, Translator, pkg_resources, json
from . import SUPPORTED_LANGUAGES, __version__
from collections import defaultdict
from copy import deepcopy
from datetime import datetime
from codecs import open
import sys
import click

try:
    from json.decoder import JSONDecodeError
except ImportError:
    pass

_RESOURCE_PACKAGE = __name__

is_python2 = sys.version_info < (3, 0)




class PoTranslator:
    """
    This is the main class of this library. This class manages all translation tasks.

    :param pot_dir: string.
        Path to the pot directory.
    :param locale_dir: string.
        Path to the locale directory.
    """
    def __init__(self, pot_dir=None, locale_dir=None):
        self.pot_dir = pot_dir
        self.locale_dir = locale_dir
        self.translator = Translator()
        return

    def translate(self, file_name, target_lang='auto', src_lang='auto', encoding='utf-8', auto_save=False, compiled=False):
        """
        Translates the given po file in the specified target language.

        :param file_name: string.
            Path to the filename of the file to translate.
        :param target_lang: string.
            Target language for translation.
        :param src_lang: string.
            Source language for translation.
        :param encoding: string.
            Encoding for saving the po files.
        :param auto_save: bool.
            Toggles auto save feature.
        :param compiled: bool.
            Toggles compilation to mo files.
        :return: tuple.
            A tuple containing the translated version of the original catalog and the status of the POFile.
        """
        po = polib.pofile(file_name, encoding=encoding)
        if target_lang == 'auto':
            try:
                target_lang = po.metadata['Language']
            except KeyError:
                raise ValueError(_('potranslator could not auto-detect the desired translation language for the file {0}.\nPlease provide a target language.').format(file_name))
        if target_lang not in SUPPORTED_LANGUAGES:
            raise ValueError(_('Unsupported language.'))
        untranslated = [elmt for elmt in po if elmt.msgstr == '' and not elmt.obsolete]
        if untranslated:
            updated = True
            try:
                translations = self.translator.translate([elmt.msgid for elmt in untranslated], src=src_lang, dest=target_lang)
                for entry, translation in zip(untranslated, translations):
                    entry.msgstr = translation.text
                po.metadata['Translated-By'] = 'potranslator {0}'.format(__version__)
                po.metadata['Last-Translator'] = 'potranslator {0}'.format(__version__)
                po.metadata['Language'] = target_lang
                po.metadata['PO-Revision-Date'] = str(datetime.today())
                print(_('{0} translations for the file {1} have been succesfully retrieved').format(SUPPORTED_LANGUAGES[target_lang], file_name))
            except JSONDecodeError as e:
                print(_('{0} translations for the file {1} could not be retrieved').format(SUPPORTED_LANGUAGES[target_lang], file_name))
            if auto_save:
                po.save(file_name)
                print(_('The file {1} has been succesfully translated in {0} and saved.').format(SUPPORTED_LANGUAGES[target_lang], file_name))
            else:
                print(_('The file {1} has been succesfully translated in {0}.').format(SUPPORTED_LANGUAGES[target_lang], file_name))
            if compiled:
                po.save_as_mofile(file_name.replace('.po', '.mo'))
        else:
            updated = False
        return po, updated

    def translate_all_locale(self, src_lang='auto', encoding='utf-8', auto_save=False, compiled=False):
        """
        Translates all the po files in the found languages in the locale folder.

        :param src_lang: string.
            Source language for translation.
        :param encoding: string.
            Encoding for saving the po files.
        :param auto_save: bool.
            Toggles auto save feature.
        :param compiled: bool.
            Toggles compilation to mo files.
        :return: Dictionary.
            A dictionary of po files.
        """
        all_locales = listdir(self.locale_dir)
        locales = [locale for locale in all_locales if locale in SUPPORTED_LANGUAGES]
        unsupported_locales = [locale for locale in all_locales if locale not in SUPPORTED_LANGUAGES]
        print(_('Attempting to translate the supported locales:\n{0}').format(', '.join(locales)))
        if unsupported_locales:
            print(_('The following locales are not yet supported by potranslator and will not be translated:\n{0}').format(', '.join(locales)))
        results = defaultdict(dict)
        for locale in locales:
            po_files = [file for file in listdir(join(self.locale_dir, locale, 'LC_MESSAGES')) if file.endswith('.po')]
            for po_file in po_files:
                path = join(self.locale_dir, locale, 'LC_MESSAGES', po_file)
                results[locale][po_file], updated = self.translate(path, src_lang=src_lang, target_lang=locale, encoding=encoding, auto_save=auto_save, compiled=compiled)
        return results

    def translate_from_pot(self, filename, status, target_langs, src_lang='auto', encoding='utf-8', auto_save=False, compiled=False):
        """
        Translates the given pot file in the specified target languages.

        :param filename: string.
            Path to the filename of the file to translate.
        :param target_langs: sequence of strings.
            Target language for translation.
        :param src_lang: string.
            Source language for translation.
        :param encoding: string.
            Encoding for saving the po files.
        :param auto_save: bool.
            Toggles auto save feature.
        :param compiled: bool.
            Toggles compilation to mo files.
        :return: Dictionary.
            A dictionary of po files.
        """
        pot = polib.pofile(filename, encoding=encoding)
        results = {}

        for target_lang in target_langs:
            po_file_name = filename.split('/')[-1].split('\\')[-1][:-1]
            po_path = join(self.locale_dir, '/'.join((target_lang, 'LC_MESSAGES', po_file_name)))
            po_dir = join(self.locale_dir, '/'.join((target_lang, 'LC_MESSAGES')))
            if not isfile(po_path):
                if not exists(po_dir):
                    makedirs(po_dir)
                po = deepcopy(pot)
                po.save(po_path)
                status['created'] += 1
                click.echo('Created: {0}'.format(po_file_name))
            results[target_lang], updated = self.translate(po_path, target_lang=target_lang, src_lang=src_lang, encoding=encoding, auto_save=auto_save, compiled=compiled)
            if updated:
                status['updated'] += 1
                click.echo('Updated: {0}'.format(po_file_name))
            else:
                status['not_changed'] +=1
                click.echo('Not Changed: {0}'.format(po_file_name))
        return results

    def translate_all_pot(self, target_langs, src_lang='auto', encoding='utf-8', auto_save=False, compiled=False):
        """
        Translates all the pot files in the pot folder in the specified target languages.

        :param target_langs: sequence of strings.
            Target language for translation.
        :param src_lang: string.
            Source language for translation.
        :param encoding: string.
            Encoding for saving the po files.
        :param auto_save: bool.
            Toggles auto save feature.
        :param compiled: bool.
            Toggles compilation to mo files.
        :return: Dictionary.
            A dictionary of po files.
        """
        pot_files = [file for file in listdir(self.pot_dir) if file.endswith('.pot')]
        results = {}
        status = {
            'created': 0,
            'updated': 0,
            'not_changed': 0,
        }
        for pot_file in pot_files:
            path = join(self.pot_dir, pot_file)
            results[pot_file] = self.translate_from_pot(path, status, target_langs=target_langs, src_lang=src_lang, encoding=encoding, auto_save=auto_save, compiled=compiled)
        return results



# -*- coding: utf-8 -*-
"""
    Python compatibility functions.

    potranslator
    ~~~~~~~~~~~
    Python package to easily translate po and pot files in any language supported by Google Translate.
    Command Line Script adapted from sphinx-intl for compatibility.

    :copyright: Copyright (c) 2018 by SekouD.
    :copyright: Copyright (c) 2013 by Takayuki SHIMIZUKAWA.
    :license: BSD, see LICENSE for details.
"""

import sys
import os
from six import PY3, exec_, b

FS_ENCODING = sys.getfilesystemencoding() or sys.getdefaultencoding()


if sys.version_info >= (2, 6):
    # Python >= 2.6
    relpath = os.path.relpath
else:  # pragma: no cover
    from os.path import curdir

    def relpath(path, start=curdir):
        """Return a relative version of a path"""
        from os.path import sep, abspath, commonprefix, join, pardir

        if not path:
            raise ValueError("no path specified")

        start_list = abspath(start).split(sep)
        path_list = abspath(path).split(sep)

        # Work out how much of the filepath is shared by start and path.
        i = len(commonprefix([start_list, path_list]))

        rel_list = [pardir] * (len(start_list)-i) + path_list[i:]
        if not rel_list:
            return start
        return join(*rel_list)
    del curdir

if PY3:
    def convert_with_2to3(filepath):  # pragma: no cover
        from lib2to3.refactor import RefactoringTool, get_fixers_from_package
        from lib2to3.pgen2.parse import ParseError
        fixers = get_fixers_from_package('lib2to3.fixes')
        refactoring_tool = RefactoringTool(fixers)
        source = refactoring_tool._read_python_source(filepath)[0]
        try:
            tree = refactoring_tool.refactor_string(source, 'conf.py')
        except ParseError:
            typ, err, tb = sys.exc_info()
            # do not propagate lib2to3 exceptions
            lineno, offset = err.context[1]
            # try to match ParseError details with SyntaxError details
            raise SyntaxError(err.msg, (filepath, lineno, offset, err.value))
        return str(tree)
else:  # pragma: no cover
    convert_with_2to3 = None


def execfile_(filepath, _globals):  # pragma: no cover
    # get config source -- 'b' is a no-op under 2.x, while 'U' is
    # ignored under 3.x (but 3.x compile() accepts \r\n newlines)
    f = open(filepath, 'rbU')
    try:
        source = f.read()
    finally:
        f.close()

    # py25,py26,py31 accept only LF eol instead of CRLF
    if sys.version_info[:2] in ((2, 5), (2, 6), (3, 1)):
        source = source.replace(b('\r\n'), b('\n'))

    # compile to a code object, handle syntax errors
    filepath_enc = filepath.encode(FS_ENCODING)
    try:
        code = compile(source, filepath_enc, 'exec')
    except SyntaxError:
        if convert_with_2to3:
            # maybe the file uses 2.x syntax; try to refactor to
            # 3.x syntax using 2to3
            source = convert_with_2to3(filepath)
            code = compile(source, filepath_enc, 'exec')
        else:
            raise

    exec_(code, _globals)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""The setup script."""

from setuptools import setup, find_packages

with open('README.rst') as readme_file:
    readme = readme_file.read()

with open('HISTORY.rst') as history_file:
    history = history_file.read()

requirements = ['Click>=6.0',
                'googletrans==2.3.0',
                'polib==1.1.0',
                'path.py==11.0.1',
                ]

setup_requirements = ['pytest-runner', ]

test_requirements = ['pip==10.0.1',
                     'pytest==3.6.2',
                     'pytest-runner==4.2',
                     'Click>=6.0',
                     'bump2version==0.5.8',
                     'wheel==0.31.1',
                     'watchdog==0.8.3',
                     'flake8==3.5.0',
                     'tox==3.0.0',
                     'coverage==4.5.1',
                     'Sphinx==1.7.5',
                     'twine==1.11.0',
                     'googletrans==2.3.0',
                     'polib==1.1.0',
                     ]

extras_require = {
    'transifex': [
        'transifex_client>=0.13.4'
    ],
}

setup(
    author="SekouD",
    author_email='sekoud.python@gmail.com',
    classifiers=[
        'Development Status :: 5 - Production/Stable',
        'Topic :: Software Development :: Libraries :: Python Modules',
        'Topic :: Utilities',
        'Intended Audience :: Developers',
        'Intended Audience :: End Users/Desktop',
        'Intended Audience :: Education',
        'Intended Audience :: Science/Research',
        'Intended Audience :: Developers',
        'Intended Audience :: Information Technology',
        'License :: OSI Approved :: BSD License',
        'Operating System :: OS Independent',
        'Operating System :: MacOS',
        'Operating System :: Microsoft',
        'Operating System :: POSIX',
        'Natural Language :: Afrikaans', 'Natural Language :: Arabic','Natural Language :: German',
        'Natural Language :: Greek', 'Natural Language :: English', 'Natural Language :: French',
        'Natural Language :: Italian', 'Natural Language :: Japanese',
        'Natural Language :: Chinese (Simplified)',
        "Programming Language :: Python :: 2",
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
    ],
    description="A python package to easily translate po and pot files in any language supported by Google Translate.",
    entry_points={
        'console_scripts': [
            'potranslator=potranslator.commands:main',
        ],
    },
    install_requires=requirements,
    license="BSD license",
    long_description=readme + '\n\n' + history,
    include_package_data=True,
    package_data={'translations': ['potranslator/locale/*'],
                  'tests': ['tests/*'],
                  'type_stubs': ['potranslator/py.typed', 'potranslator/*']},
    keywords='potranslator sphinx sphinx-intl gettext localization translation translate po pot mo internationalization python google',
    name='potranslator',
    packages=find_packages(include=['potranslator']),
    setup_requires=setup_requirements,
    test_suite='tests',
    tests_require=test_requirements,
    url='https://github.com/SekouD/potranslator',
    version='1.0.4',
    zip_safe=False,
)



# -*- coding: utf-8 -*-
"""
    potranslator
    ~~~~~~~~~~~
    Python package to easily translate po and pot files in any language supported by Google Translate.
    Command Line Script adapted from sphinx-intl for compatibility.

    :copyright: Copyright (c) 2018 by SekouD.
    :copyright: Copyright (c) 2013 by Takayuki SHIMIZUKAWA.
    :license: BSD, see LICENSE for details.
"""

from __future__ import with_statement

import sys
import re
import os
from glob import glob

import click

from . import basic
from . import transifex
from .pycompat import execfile_, relpath

ENVVAR_PREFIX = 'POTRANSLATOR'


# ==================================
# utility functions

class Tags(object):
    def __init__(self, tags=None):
        # type: (List[unicode]) -> None
        self.tags = dict.fromkeys(tags or [], True)

    def has(self, tag):
        # type: (unicode) -> bool
        return tag in self.tags

    __contains__ = has

    def __iter__(self):
        # type: () -> Iterator[unicode]
        return iter(self.tags)

    def add(self, tag):
        # type: (unicode) -> None
        self.tags[tag] = True

    def remove(self, tag):
        # type: (unicode) -> None
        self.tags.pop(tag, None)


def read_config(path, passed_tags):

    tags = Tags()
    passed_tags = sum(passed_tags, ())
    for tag in passed_tags:
        tags.add(tag)

    namespace = {
        "__file__": os.path.abspath(path),
        "tags": tags,
    }

    olddir = os.getcwd()
    try:
        if not os.path.isfile(path):
            msg = "'%s' was not found (or specify --locale-dir option)." % path
            raise click.BadParameter(msg)
        os.chdir(os.path.dirname(path) or ".")
        execfile_(os.path.basename(path), namespace)
    finally:
        os.chdir(olddir)

    return namespace


def get_lang_dirs(path):
    dirs = [relpath(d, path)
            for d in glob(path+'/[a-z]*')
            if os.path.isdir(d) and not d.endswith('pot')]
    return (tuple(dirs),)


# ==================================
# click options

class LanguagesType(click.ParamType):
    name = 'languages'
    envvar_list_splitter = ','

    def convert(self, value, param, ctx):
        langs = value.split(',')
        return tuple(langs)


LANGUAGES = LanguagesType()


class TagsType(click.ParamType):
    name = 'tags'
    envvar_list_splitter = ','

    def convert(self, value, param, ctx):
        tags = value.split(',')
        return tuple(tags)


TAGS = TagsType()


option_locale_dir = click.option(
    '-d', '--locale-dir',
    envvar=ENVVAR_PREFIX + '_LOCALE_DIR',
    type=click.Path(exists=False, file_okay=False),
    default='locales', metavar='<DIR>', show_default=True,
    help='locale directories that allow comma separated string. This option '
         'overrides locale_dir in conf.py setting if provided. Default is empty '
         'list.')

option_pot_dir = click.option(
    '--pot-dir', '-p',
    envvar=ENVVAR_PREFIX + '_POT_DIR',
    type=click.Path(exists=False, file_okay=False),
    metavar='<DIR>', show_default=True,
    help="pot files directory which is generated by sphinx. "
         "Default is 'pot' directory under '--locale-dir' path.")

option_output_dir = click.option(
    '--output-dir', '-o',
    envvar=ENVVAR_PREFIX + '_OUTPUT_DIR',
    type=click.Path(exists=False, file_okay=False),
    metavar='<DIR>', show_default=True,
    help="mo files directory where files are written. "
         "Default is to match the '--locale-dir' path.")

option_tag = click.option(
    '-t', '--tag',
    envvar=ENVVAR_PREFIX + '_TAG',
    type=TAGS, default=(), metavar='<TAG>', show_default=True,
    multiple=True,
    help="Pass tags to conf.py, as same as passed to sphinx-build -t option.")

option_language = click.option(
    '-l', '--language',
    envvar=ENVVAR_PREFIX + '_LANGUAGE',
    type=LANGUAGES, default=(), metavar='<LANG>', show_default=True,
    multiple=True,
    help="Target language to update po files. Default is ALL.")

option_transifex_username = click.option(
    '--transifex-username',
    envvar=ENVVAR_PREFIX + '_TRANSIFEX_USERNAME',
    type=str, metavar='<USERNAME>', show_default=True,
    help="Your transifex username. Default is None.")

option_transifex_password = click.option(
    '--transifex-password',
    envvar=ENVVAR_PREFIX + '_TRANSIFEX_PASSWORD',
    type=str, metavar='<PASSWORD>', show_default=True,
    help="Your transifex password. Default is None.")

option_transifex_project_name = click.option(
    '--transifex-project-name',
    envvar=ENVVAR_PREFIX + '_TRANSIFEX_PROJECT_NAME',
    type=str, metavar='<PROJECT-NAME>', show_default=True,
    help="Your transifex project name. default is None")

CONTEXT_SETTINGS = dict(help_option_names=['-h', '--help'])

# ==================================
# commands

@click.group(context_settings=CONTEXT_SETTINGS)
@click.option(
    '-c', '--config',
    type=click.Path(exists=True, file_okay=True, dir_okay=False),
    default=None, metavar='<FILE>',
    help='Sphinx conf.py file to read a locale directory setting.')
@option_tag
@click.pass_context
def main(ctx, config, tag):
    """
    Environment Variables:
    All command-line options can be set with environment variables using the
    format POTRANSLATOR_<UPPER_LONG_NAME> . Dashes (-) have to replaced with
        underscores (_).

    For example, to set the languages:

    export POTRANSLATOR_LANGUAGE=de,ja

    This is the same as passing the option to txutil directly:

    potranslator update --language=de --language=ja
    """
    # load conf.py
    ctx.config = config
    if ctx.config is None:
        for c in ('conf.py', 'source/conf.py'):
            if os.path.exists(c):
                ctx.config = c
                break

    # for locale_dir
    ctx.locale_dir = None
    if ctx.config:  # pragma: no cover
        cfg = read_config(ctx.config, tag)
        if 'locale_dirs' in cfg:
            ctx.locale_dir = os.path.join(
                os.path.dirname(ctx.config), cfg['locale_dirs'][0])

    # for pot_dir
    ctx.pot_dir = None
    for d in ('_build/gettext', 'build/gettext',
              '_build/locale', 'build/locale'):
        if os.path.exists(d):
            ctx.pot_dir = d
            break

    # for transifex_project_name
    ctx.transifex_project_name = None
    target = os.path.normpath('.tx/config')
    if os.path.exists(target):
        matched = re.search(r'\[(.*)\..*\]', open(target, 'r').read())
        if matched:
            ctx.transifex_project_name = matched.groups()[0]
            click.echo(
                'Project name loaded from .tx/config: {0}'.format(
                    ctx.transifex_project_name))

    ctx.default_map = {
        'update': {
            'locale_dir': ctx.locale_dir,
            'pot_dir': ctx.pot_dir,
        },
        'build': {
            'locale_dir': ctx.locale_dir,
        },
        'stat': {
            'locale_dir': ctx.locale_dir,
        },
        'update-txconfig-resources': {
            'locale_dir': ctx.locale_dir,
            'pot_dir': ctx.pot_dir,
            'transifex_project_name': ctx.transifex_project_name,
        },
    }


@main.command(context_settings=CONTEXT_SETTINGS)
@option_locale_dir
@option_pot_dir
@option_language
def update(locale_dir, pot_dir, language):
    """
    Update specified language's po files from pot.

    \b
    For examples:
       potranslator update -l de -l ja
       potranslator update -l de,ja
    """
    if not pot_dir:
        pot_dir = os.path.join(locale_dir, 'pot')
    if not os.path.exists(pot_dir):
        msg = ("%(pot_dir)r does not exist. Please specify a pot directory with "
               "-p option, or declare your pot files in %(pot_dir)r."
               % locals())
        raise click.BadParameter(msg, param_hint='pot_dir')
    if not language:
        language = get_lang_dirs(locale_dir)
    languages = sum(language, ())  # flatten
    if not languages:
        msg = ("No languages were found. Please specify a language with -l "
               "option, or declare language directories under %(locale_dir)r "
               "directory."
               % locals())
        raise click.BadParameter(msg, param_hint='language')

    basic.update(locale_dir, pot_dir, languages)


@main.command(context_settings=CONTEXT_SETTINGS)
@option_locale_dir
@option_output_dir
@option_language
def build(locale_dir, output_dir, language):
    """
    Update specified language's po files from pot.
    """
    if not language:
        language = get_lang_dirs(locale_dir)
    languages = sum(language, ())  # flatten

    if not output_dir or (
            os.path.exists(output_dir) and
            os.path.samefile(locale_dir, output_dir)):
        output_dir = locale_dir

    basic.build(locale_dir, output_dir, languages)


@main.command(context_settings=CONTEXT_SETTINGS)
@option_locale_dir
@option_language
def stat(locale_dir, language):
    """
    Print statistics for all po files.
    """
    if not language:
        language = get_lang_dirs(locale_dir)
    languages = sum(language, ())  # flatten
    basic.stat(locale_dir, languages)


@main.command('create-transifexrc')
@option_transifex_username
@option_transifex_password
def create_transifexrc(transifex_username, transifex_password):
    """
    Create `$HOME/.transifexrc`
    """
    transifex.create_transifexrc(transifex_username, transifex_password)


@main.command('create-txconfig')
def create_txconfig():
    """
    Create `./.tx/config`
    """
    transifex.create_txconfig()


@main.command('update-txconfig-resources')
@option_transifex_project_name
@option_locale_dir
@option_pot_dir
def update_txconfig_resources(transifex_project_name, locale_dir, pot_dir):
    """
    Update resource sections of `./.tx/config`.
    """
    if not pot_dir:
        pot_dir = os.path.join(locale_dir, 'pot')

    transifex.update_txconfig_resources(transifex_project_name, locale_dir, pot_dir)


if __name__ == '__main__':
    sys.exit(main(auto_envvar_prefix=ENVVAR_PREFIX))
